/*
 *
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago             	Data di creazione : 23/12/98
 * Codice   :
 *
 *
 ******************************* Revisione file *******************************
 *-------------------------------------|--------------------------------------*
 * Revisore : 								Data di revisione :
 * Versione :
 *
 * Note:
 *
 */

#include	"stdafx.h"

int		TBarcodeQQueue::GetCount()
{
	TLock lock(this);

	return	dw_count;
}

bool	TBarcodeQQueue::Insert(const TBarcodeQItem& item)
{
	TLock lock(this);
	
	TBarcodeQItem*	p	=	new	TBarcodeQItem;

	(tail	?	tail->next	: head)	=	p;

	*p	=	item;

	tail	=	p;

	dw_count++;

	return	true;
}

bool	TBarcodeQQueue::Remove(TBarcodeQItem& item)
{
    TLock lock(this);

	TBarcodeQItem*	p;

	if(tail==NULL)
		return	false;

	p	=	head;

	item	=	*head;

	item.next	=	NULL;

	if(p	==	tail)
		tail	=	NULL;
	else
		head	=	p->next;

	delete p;

	dw_count--;
	
	if(dw_count==0)
		head	=	tail	=	0;

	return true;
}

TBarcodeQItem*	TBarcodeQQueue::findPrev(TBarcodeQItem* item)
{
    TLock lock(this);

	if(item==head)
		return	NULL;

	TBarcodeQItem*	cursor=head;

	while( !item->IsEqual(cursor->next) )
        cursor = cursor->next;
    
    return cursor;
}

void	TBarcodeQQueue::Detach(TBarcodeQItem* toDetach)
{
    TLock lock(this);

	TBarcodeQItem*	prev=findPrev(toDetach);
	
	if(prev==NULL)
	{
		if(toDetach->next)
			head	=	toDetach->next;
		else
			head	=	tail;

		delete	toDetach;

		dw_count--;

		if(dw_count==0)
			head	=	tail	=	0;

		return;
	}

	if(prev->next == toDetach)
	{
		prev->next =	prev->next->next;
		
		if(prev->next==NULL)
			tail	=	prev;

		dw_count--;

		if(dw_count==0)
			head	=	tail	=	0;

		delete	toDetach;
	}
}

bool	TBarcodeQQueue::Find(const	TBarcodeQItem& item, BYTE by_Status, BOOL b_Remove)
{
    TLock lock(this);

	TBarcodeQItem*	cursor=head;

	while(cursor)
	{
		if(*cursor	==	item && cursor->by_status == by_Status)
		{
			if(b_Remove)
			{
				Detach(cursor);
			}
			return	true;
			
		}
		
		cursor=cursor->next;
	}

	return	false;
}

bool	TBarcodeQQueue::ChangeTail(const	TBarcodeQItem& item)
{
    TLock lock(this);

	if(tail==NULL)
		return	false;

	*tail	=	item;

	return	true;
}


bool	TBarcodeQQueue::ChangeHead(const	TBarcodeQItem& item)
{
    TLock lock(this);

	if(tail==NULL)
		return	false;

	*head	=	item;

	return	true;
}


void	TBarcodeQQueue::Flush()
{
    TLock lock(this);

	TBarcodeQItem	i;
	
	while(Remove(i));
}

bool	TBarcodeQQueue::PeekHead(TBarcodeQItem& item)
{
    TLock lock(this);

	if(tail==NULL)
		return	false;

	item	=	*head;

	return	true;
}

void	TBarcodeQQueue::ForEach( void (*ccAppFunc)( const	TBarcodeQItem* item, void * ), void *arg )
{
    TLock lock(this);

	TBarcodeQItem*	cursor=head;

	while(cursor)
	{
		ccAppFunc(cursor, arg);
		
		cursor=cursor->next;
	}
}

bool	TBarcodeQQueue::checkHeader(FILE* file)
{
	char	sz_buffer[ci_BARCODEQUEUEHEADERLEN];

	if(fread(sz_buffer, 1, sizeof(sz_buffer), file))
	{
		if(!memcmp(sz_buffer, TBarcodeQQueue::sz_fileHeader, sizeof(sz_buffer)))
			return	true;
	}

	return	false;
}


bool	TBarcodeQQueue::LoadFromFile(char*	sz_filename, bool	b_flushPreviuos)
{
    TLock lock(this);

	if(b_flushPreviuos)
		Flush();

	FILE*	file;

	if(NULL!=(file	=	fopen(sz_filename, "rb")))
	{
		if(checkHeader(file))
		{
			DWORD	dw_FileCount;
			
			fread(&dw_FileCount, 1, sizeof(dw_FileCount), file);

			for(DWORD	dw_i=0;dw_i<dw_FileCount;dw_i++)
			{
				TBarcodeQItem		item;

				fread(&item, 1, sizeof(item), file);

				Insert(item);
			}

			return	true;
		}
	}
	
	return	false;
}

void	writeItem(const	TBarcodeQItem* item, void *arg)
{
	FILE*	p_file=	(FILE*)(arg);

	fwrite(item, 1, sizeof(*item), p_file);
}

char	TBarcodeQQueue::sz_fileHeader[ci_BARCODEQUEUEHEADERLEN]	=	{	'A', 'X', 'B', 'C', 'Q', '1'	};

bool	TBarcodeQQueue::writeHeader(FILE* file)
{
	return	(fwrite(TBarcodeQQueue::sz_fileHeader, 1, sizeof(TBarcodeQQueue::sz_fileHeader), file)	>	0) ? true : false;
}

bool	TBarcodeQQueue::WriteToFile(char*	sz_filename)
{
    TLock lock(this);

	FILE*	file;

	if(NULL!=(file	=	fopen(sz_filename, "wb")))
	{
		writeHeader(file);

		fwrite(&dw_count, 1, sizeof(dw_count), file);

		ForEach(writeItem, file);

		fclose(file);

		return	true;
	}

	return	false;
}

